import es
import lotterylib

class ImproperlyConfigured(Exception):
    """
    Exception raised if you use a obviously bad API key or server key.
    """
    
def load_config():
    """
    The layout of our config file is dead simple:
    If the line starts with 'api key:' (case insensitive) the stuff after the
    colon (whitespaces stripped) is considered the api key.
    The same goes for 'server key:' for the server key respectively.
    The default file uses JavaScript style comments so as not to confuse the 
    end user.
    We also do a very simple regex-check of the key if it's possibly correct.
    """
    import path
    import re
    key_pattern = re.compile(r'[a-zA-Z0-9]{64}')
    cfgfile = path.path(__file__).dirname().joinpath('keys.txt')
    lines = cfgfile.lines(retain=False)
    api_key = ''
    server_key = ''
    for line in lines:
        if ':' in line:
            start, end = line.split(':', 1)
            if start.lower().strip() == 'api key':
                api_key = end.strip()
            elif start.lower().strip() == 'server key':
                server_key = end.strip()
    if not key_pattern.match(api_key):
        raise ImproperlyConfigured, "Invalid API Key: '%s'" % api_key
    if not key_pattern.match(server_key):
        raise ImproperlyConfigured, "Invalid Server Key: '%s'" % server_key
    return api_key, server_key
api_key, server_key = load_config()

#CONSTANT! DO NOT CHANGE:
APP_KEY = 'J1e7Dbe0VkHJvu80zzcrXQAVMxt8anYELqYr3vVkiYaNagv8IsA2YvGWgamZMkpz'

app = lotterylib.Application(api_key, server_key, APP_KEY)

srpg = es.import_addon('sourcerpg')

class Cache(object):
    """
    Cache srpg player objects
    """
    def __init__(self):
        self.reset()
        
    def reset(self):
        self.players = {}
        
    def getSRPGPlayer(self, steamid):
        if steamid not in self.players:
            try:
                userid = es.getuserid(steamid)
            except:
                userid = None
            self.players[steamid] = srpg.players.players.get(userid, None)
        return self.player[steamid]
cache = Cache()

def db_update(steamid, key, amount):
    """
    Update the data in the database directly.
    """
    # @311: def getUserIdFromSteamId(self, steamId):
    dbid = srpg.database.getUserIdFromSteamId(steamid)
    if not dbid:
        # @299: def addPlayer(self, steamid, name):
        dbid = srpg.database.addPlayer(steamid, 'lottery winner')
    # @425: def increment(self, table, primaryKeyName, primaryKeyValue, options):
    srpg.database.increment('Player', 'UserID', dbid, {key: amount})

def convert(item):
    """
    Convert stuff like '500 XP' to (500, 'xp')
    """
    amount, item_type = item.split(' ')
    return int(amount), item_type.lower()
    
def add_xp(steamid, amount):
    """
    Add `amount` XP for the player `steamid`
    """
    srpgplayer = cache.getSRPGPlayer(steamid)
    # check if player is on the server
    if srpgplayer:
        srpgplayer.addXp(amount, "You won the lottery on http://esl.ojii.ch! Gratulations!")
    else:
        db_update(steamid, 'xp', amount)
        
def add_credits(steamid, amount):
    """
    Add `amount` credits for the player `steamid`
    """
    srpgplayer = cache.getSRPGPlayer(steamid)
    # check if player is on the server
    if srpgplayer:
        srpgplayer['credits'] += amount
    else:
        db_update(steamid, 'credits', amount)

def response_handler(response):
    """
    Handle the lotterylib response `response`
    """
    if response.error:
        # The return is only here to show that this if clause would 'end' the function
        return response.raise_error()
    cache.reset()
    # iterate over winitems
    for item, winners in response.result.iteritems():
        # convert the item name to usable stuff
        amount, item_type = convert(item)
        if item_type == 'xp':
            # iterate over winners
            for winner in winners:
                add_xp(winner, amount)
        elif item_type == 'credits':
            # iterate over winners
            for winner in winners:
                add_credits(winner, amount)

def es_map_start(ev):
    app.get_queue(response_handler)
